@@ -16,11 +16,11 @@
 
 import zipfile
 import six
-import mimetypes
 import logging
 import uuid
 import posixpath as zip_path
 import os.path
+from collections import OrderedDict
 
 try:
     from urllib.parse import unquote
@@ -31,12 +31,7 @@ from lxml import etree
 
 import ebooklib
 
-from ebooklib.utils import parse_string, parse_html_string
-
-
-# This really should not be here
-mimetypes.init()
-mimetypes.add_type('application/xhtml+xml', '.xhtml')
+from ebooklib.utils import parse_string, parse_html_string, guess_type
 
 
 # Version of EPUB library
@@ -239,11 +234,12 @@ class EpubHtml(EpubItem):
     """
     _template_name = 'chapter'
 
-    def __init__(self, uid=None, file_name='', media_type='', content=None, title='', lang=None):
+    def __init__(self, uid=None, file_name='', media_type='', content=None, title='', lang=None, direction=None):
         super(EpubHtml, self).__init__(uid, file_name, media_type, content)
 
         self.title = title
         self.lang = lang
+        self.direction = direction
 
         self.links = []
         self.properties = []
@@ -325,12 +321,12 @@ class EpubHtml(EpubItem):
 
     def get_body_content(self):
         """
-        Returns content of BODY element for this HTML document. Content will be of type 'str' (Python 2) or 'bytes' (Python 3).
+        Returns content of BODY element for this HTML document. Content will be of type 'str' (Python 2)
+        or 'bytes' (Python 3).
 
         :Returns:
           Returns content of this document.
         """
-        content = self.get_content()
 
         try:
             html_tree = parse_html_string(self.content)
@@ -356,7 +352,8 @@ class EpubHtml(EpubItem):
 
     def get_content(self, default=None):
         """
-        Returns content for this document as HTML string. Content will be of type 'str' (Python 2) or 'bytes' (Python 3).
+        Returns content for this document as HTML string. Content will be of type 'str' (Python 2)
+        or 'bytes' (Python 3).
 
         :Args:
           - default: Default value for the content if it is not defined.
@@ -408,6 +405,8 @@ class EpubHtml(EpubItem):
         # create and populate body
 
         _body = etree.SubElement(tree_root, 'body')
+        if self.direction:
+            _body.set('dir', self.direction)
 
         body = html_tree.find('body')
         if body is not None:
@@ -535,6 +534,7 @@ class EpubBook(object):
 
         self.title = ''
         self.language = 'en'
+        self.direction = None
 
         self.templates = {
             'ncx': NCX_XML,
@@ -543,7 +543,9 @@ class EpubBook(object):
             'cover': COVER_XML
         }
 
-        self.add_metadata('OPF', 'generator', '', {'name': 'generator', 'content': 'Ebook-lib %s' % '.'.join([str(s) for s in VERSION])})
+        self.add_metadata('OPF', 'generator', '', {
+            'name': 'generator', 'content': 'Ebook-lib %s' % '.'.join([str(s) for s in VERSION])
+            })
 
         # default to using a randomly-unique identifier if one is not specified manually
         self.set_identifier(str(uuid.uuid4()))
@@ -589,6 +591,14 @@ class EpubBook(object):
 
         self.add_metadata('DC', 'language', lang)
 
+    def set_direction(self, direction):
+        """
+        :Args:
+          - direction: Options are "ltr", "rtl" and "default"
+        """
+
+        self.direction = direction
+
     def set_cover(self, file_name, content, create_page=True):
         """
         Set cover and create cover document if needed.
@@ -608,7 +618,7 @@ class EpubBook(object):
             c1 = EpubCoverHtml(image_name=file_name)
             self.add_item(c1)
 
-        self.add_metadata(None, 'meta', '', {'name': 'cover', 'content': 'cover-img'})
+        self.add_metadata(None, 'meta', '', OrderedDict([('name', 'cover'), ('content', 'cover-img')]))
 
     def add_author(self, author, file_as=None, role=None, uid='creator'):
         "Add author for this document"
@@ -666,7 +676,7 @@ class EpubBook(object):
           - item: Item instance
         """
         if item.media_type == '':
-            (has_guessed, media_type) = mimetypes.guess_type(item.get_name().lower())
+            (has_guessed, media_type) = guess_type(item.get_name().lower())
 
             if has_guessed:
                 if media_type is not None:
@@ -812,7 +822,9 @@ class EpubWriter(object):
     DEFAULT_OPTIONS = {
         'epub2_guide': True,
         'epub3_landmark': True,
-        'landmark_title': 'Guide'
+        'landmark_title': 'Guide',
+        'spine_direction': True,
+        'package_direction': False
     }
 
     def __init__(self, name, book, options=None):
@@ -840,10 +852,13 @@ class EpubWriter(object):
         self.out.writestr(CONTAINER_PATH, container_xml)
 
     def _write_opf_file(self):
-        root = etree.Element('package',
-                             {'xmlns': NAMESPACES['OPF'],
+        package_attributes = {'xmlns': NAMESPACES['OPF'],
                               'unique-identifier': self.book.IDENTIFIER_ID,
-                              'version': '3.0'})
+                              'version': '3.0'}
+        if self.book.direction and self.options['package_direction']:
+            package_attributes['dir'] = self.book.direction
+
+        root = etree.Element('package', package_attributes)
 
         prefixes = ['rendition: http://www.idpf.org/vocab/rendition/#'] + self.book.prefixes
         root.attrib['prefix'] = ' '.join(prefixes)
@@ -933,7 +948,11 @@ class EpubWriter(object):
                 etree.SubElement(manifest, 'item', opts)
 
         # SPINE
-        spine = etree.SubElement(root, 'spine', {'toc': _ncx_id or 'ncx'})
+        spine_attributes = {'toc': _ncx_id or 'ncx'}
+        if self.book.direction and self.options['spine_direction']:
+            spine_attributes['page-progression-direction'] = self.book.direction
+
+        spine = etree.SubElement(root, 'spine', spine_attributes)
 
         for _item in self.book.spine:
             # this is for now
@@ -1005,8 +1024,8 @@ class EpubWriter(object):
 
     def _get_nav(self, item):
         # just a basic navigation for now
-        ncx = parse_string(self.book.get_template('nav'))
-        root = ncx.getroot()
+        nav_xml = parse_string(self.book.get_template('nav'))
+        root = nav_xml.getroot()
 
         root.set('lang', self.book.language)
         root.attrib['{%s}lang' % NAMESPACES['XML']] = self.book.language
@@ -1019,7 +1038,9 @@ class EpubWriter(object):
 
         # for now this just handles css files and ignores others
         for _link in item.links:
-            _lnk = etree.SubElement(head, 'link', {"href": _link.get('href', ''), "rel": "stylesheet", "type": "text/css"})
+            _lnk = etree.SubElement(head, 'link', {
+                "href": _link.get('href', ''), "rel": "stylesheet", "type": "text/css"
+                })
 
         body = etree.SubElement(root, 'body')
         nav = etree.SubElement(body, 'nav', {'{%s}type' % NAMESPACES['EPUB']: 'toc', 'id': 'id'})
@@ -1050,7 +1071,6 @@ class EpubWriter(object):
                     a.text = item.title
                 elif isinstance(item, EpubHtml):
                     li = etree.SubElement(ol, 'li')
-
                     a = etree.SubElement(li, 'a', {'href': os.path.relpath(item.file_name, nav_dir_name)})
                     a.text = item.title
 
@@ -1087,10 +1107,13 @@ class EpubWriter(object):
                     _title = elem.get('title', '')
 
                 guide_type = elem.get('type', '')
-                a_item = etree.SubElement(li_item, 'a', {'{%s}type' % NAMESPACES['EPUB']: guide_to_landscape_map.get(guide_type, guide_type), 'href': os.path.relpath(_href, nav_dir_name)})
+                a_item = etree.SubElement(li_item, 'a', {
+                    '{%s}type' % NAMESPACES['EPUB']: guide_to_landscape_map.get(guide_type, guide_type),
+                    'href': os.path.relpath(_href, nav_dir_name)
+                    })
                 a_item.text = _title
 
-        tree_str = etree.tostring(root, pretty_print=True, encoding='utf-8', xml_declaration=True)
+        tree_str = etree.tostring(nav_xml, pretty_print=True, encoding='utf-8', xml_declaration=True)
 
         return tree_str
 
@@ -1124,7 +1147,9 @@ class EpubWriter(object):
                 if isinstance(item, tuple) or isinstance(item, list):
                     section, subsection = item[0], item[1]
 
-                    np = etree.SubElement(itm, 'navPoint', {'id': section.get_id() if isinstance(section, EpubHtml) else 'sep_%d' % uid})
+                    np = etree.SubElement(itm, 'navPoint', {
+                        'id': section.get_id() if isinstance(section, EpubHtml) else 'sep_%d' % uid
+                        })
                     nl = etree.SubElement(np, 'navLabel')
                     nt = etree.SubElement(nl, 'text')
                     nt.text = section.title
@@ -1145,7 +1170,7 @@ class EpubWriter(object):
                     _parent = itm
                     _content = _parent.find('content')
 
-                    if _content != None:
+                    if _content is not None:
                         if _content.get('src') == '':
                             _content.set('src', item.href)
 
@@ -1159,7 +1184,7 @@ class EpubWriter(object):
                     _parent = itm
                     _content = _parent.find('content')
 
-                    if _content != None:
+                    if _content is not None:
                         if _content.get('src') == '':
                             _content.set('src', item.file_name)
 
@@ -1337,7 +1362,7 @@ class EpubReader(object):
                 elif 'cover' in properties:
                     ei = EpubCoverHtml()
 
-                    ei.content = self.read_file(os_path.join(self.opf_dir, unquote(r.get('href'))))
+                    ei.content = self.read_file(zip_path.join(self.opf_dir, unquote(r.get('href'))))
                 else:
                     ei = EpubHtml()
 
@@ -1368,7 +1393,6 @@ class EpubReader(object):
                 ei.media_type = media_type
 
                 ei.content = self.read_file(zip_path.join(self.opf_dir, ei.get_name()))
-              # r.get('properties')
 
             self.book.add_item(ei)
 
@@ -1381,7 +1405,6 @@ class EpubReader(object):
         def _get_children(elems, n, nid):
             label, content = '', ''
             children = []
-            _id = ''
 
             for a in elems.getchildren():
                 if a.tag == '{%s}navLabel' % NAMESPACES['DAISY']:
@@ -1398,7 +1421,7 @@ class EpubReader(object):
                 return (Section(label, href=content),
                         children)
             else:
-                return (Link(content, label, nid))
+                return Link(content, label, nid)
 
         self.book.toc = _get_children(nav_map, 0, '')
 
@@ -1439,13 +1462,14 @@ class EpubReader(object):
         self.book.spine = [(t.get('idref'), t.get('linear', 'yes')) for t in spine]
 
         toc = spine.get('toc', '')
+        self.book.set_direction(spine.get('page-progression-direction', None))
 
         # should read ncx or nav file
         if toc:
             try:
                 ncxFile = self.read_file(zip_path.join(self.opf_dir, self.book.get_item_with_id(toc).get_name()))
             except KeyError:
-                raise EpubError(-1, 'Can not find ncx file.')
+                raise EpubException(-1, 'Can not find ncx file.')
 
             self._parse_ncx(ncxFile)
 
@@ -1458,7 +1482,7 @@ class EpubReader(object):
         try:
             s = self.read_file(self.opf_file)
         except KeyError:
-            raise EpubError(-1, 'Can not find container file')
+            raise EpubException(-1, 'Can not find container file')
 
         self.container = parse_string(s)
 
